import sys
from woodcut.project import Project
from optparse import OptionParser

import pkg_resources
version = pkg_resources.require("woodcut")[0].version

def main():
    usage = "usage: %prog [options] command src_path build_path"
    parser = OptionParser(usage=usage, version="%prog {0}".format(version))
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False, help="Show more logging information")
    parser.add_option("-c", "--copy", action="store_true", dest="copy", default=False, help="Copy non-template files instead of symlinking them")
    (options, args) = parser.parse_args()
    
    if len(args) == 3 and args[0] == 'build':
        Project(src_root=args[1], build_root=args[2], copy=options.copy).build()
    elif len(args) == 3 and args[0] == 'clean':
        Project(src_root=args[1], build_root=args[2], copy=options.copy).clean()
    else:
        parser.print_usage()
    
    return 0


from setuptools import setup

setup(
    name = "woodcut",
    version = "0.4.1",
    author = "Luke Cyca",
    author_email = "me@lukecyca.com",
    description = ("Minimalist content management system for static websites."),
    license = "BSD-new",
    keywords = "mako template static web development",
    url = "http://wiki.github.com/lukecyca/woodcut",
    long_description="Woodcut is a system for building static websites from Mako source files. It will walk your source directory, process any templates it finds, and produce a complete website in the build directory, ready to rsync to your webserver.",
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Natural Language :: English",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
        "Topic :: Text Processing :: Markup",
    ],
    
    packages=['woodcut'],
    install_requires=["Mako>=0.4.2"],
    
    entry_points = {
        'console_scripts': [
            'woodcut = woodcut:main',
        ]
    }
)



#!/usr/bin/python

import os
import os.path
from shutil import copy
from datetime import datetime
from mako.template import Template
from mako.lookup import TemplateLookup
from mako.exceptions import text_error_template, SyntaxException, CompileException, TemplateLookupException


IGNORE_FILES = ['.DS_Store', '.hgignore', '.gitignore']
IGNORE_DIRECTORIES = ['templates', '.hg', '.git', '.mako_modules']

class Project(object):
    def __init__(self, src_root, build_root, **kwargs):
        self.src_root = os.path.abspath(src_root)
        self.build_root = os.path.abspath(build_root)
        
        try:
            self.copy_flag = kwargs['copy']
        except KeyError:
            self.copy_flag = False
        
        self.lookup = TemplateLookup(directories=[self.src_root],
                                     module_directory=os.path.join(self.src_root, '.mako_modules'),
                                     output_encoding='utf-8',
                                     input_encoding='utf-8',
                                     )
        
    
    def build_file(self, root_relative_src_path):
        def relative_path(path):
            """Translates a root-relative path to a path relative to this file"""
            backout_path = os.path.relpath('.', os.path.dirname(root_relative_src_path))
            return os.path.normpath(os.path.join(backout_path, path))
        
        # Absolute paths to source and build objects
        src_path = os.path.normpath(os.path.join(self.src_root, root_relative_src_path))
        build_path = os.path.normpath(os.path.join(self.build_root, root_relative_src_path))
        
        if os.path.exists(build_path):
            os.unlink(build_path)
        
        extension = os.path.splitext(src_path)[1]
        
        # Skip non-templates, and just link them to the source file
        if extension not in ['.mako']:
            if self.copy_flag:
                print "Copying %s" % root_relative_src_path
                copy(src_path, build_path)
            else:
                print "Symlinking %s" % root_relative_src_path
                os.symlink(src_path, build_path)
            return
        
        # Determine output filename
        build_path = build_path.replace('.mako','')
        
        # Render the template to the output path
        print "Rendering %s" % root_relative_src_path.replace('.mako','')

        with open(build_path, 'w') as fh:
            try:
                mako_src = self.lookup.get_template(root_relative_src_path)
                fh.write(mako_src.render(relative_path=relative_path))
            except (CompileException, TemplateLookupException, SyntaxException), e:
                print '  {0.__class__.__name__}: {0}'.format(e)
        

    def build(self):
        if not os.path.exists(self.build_root):
            os.mkdir(self.build_root)
        os.chdir(self.src_root)
        for path, dirs, files in os.walk('.'):
            for d in list(dirs):
                if d in IGNORE_DIRECTORIES:
                    dirs.remove(d)
                elif not os.path.exists(os.path.join(self.build_root, path, d)):
                    os.mkdir(os.path.join(self.build_root, path, d))
            for f in files:
                if f not in IGNORE_FILES:
                    self.build_file(os.path.join(path, f))

    def clean(self):
        """Delete everything in the build directory"""
        for root, dirs, files in os.walk(self.build_root, topdown=False):
            for f in files:
                os.remove(os.path.join(root,f))
            for d in dirs:
                os.rmdir(os.path.join(root,d))
                